import os
import shutil

from hdfs import InsecureClient
from .file_io import FileIO
from .get_arguments import get_argument
from zipfile import ZipFile


class Hdfs(FileIO):

    storage_type = os.path.basename(__file__).split('.py')[0]
    
    def __init__(self, storage_type=None):
        super().__init__()
        self.web_hdfs_url = get_argument('webHdfsUrl')

    def upload(self, local_path, remote_path, overwrite=True):

        output_zip = remote_path.split("/")[-1] + ".zip"
        self.zip_file(local_path, output_zip)

        # Upload model and zipped model to HDFS
        client = InsecureClient(self.web_hdfs_url)
        client.upload(remote_path, local_path, overwrite=overwrite, temp_dir="/tmp")
        client.upload(remote_path + ".zip", output_zip, overwrite=overwrite, temp_dir="/tmp")

    def download(self, remote_path, local_path, overwrite=True):
        # Dowload model from HDFS to disk
        client = InsecureClient(self.web_hdfs_url)
        client.download(remote_path, local_path, overwrite=overwrite, temp_dir="/tmp")

    @staticmethod
    def zip_file(local_path, output_zip):

        # Zip the file/directory to upload

        if os.path.isdir(local_path):

            shutil.make_archive(local_path, 'zip', local_path)

        else:  # it's a file

            ZipFile(output_zip, mode='w').write(local_path)



import os
from .file_io import FileIO
from distutils.dir_util import copy_tree
from distutils.errors import DistutilsFileError
from distutils.file_util import copy_file


class Filesystem(FileIO):

    storage_type = os.path.basename(__file__).split('.py')[0]
    
    def __init__(self, storage_type=None):
        super().__init__()

    def upload(self, local_path, remote_path, overwrite=True):
        self.copy_tree_or_file(local_path, remote_path)
        print("Faking upload because it's running locally.")

    def download(self, remote_path, local_path, overwrite=True):
        self.copy_tree_or_file(remote_path, local_path)
        print("Faking download because it's running locally.")

    def copy_tree_or_file(self, from_path, to_path):
        if not os.path.exists(from_path):
            raise Exception("File or folder not present! Be sure that you're saving your file (model?) correctly.")
        # If it's a folder (compliant with older versions of python, now better ways are present).
        try:
            copy_tree(from_path, to_path)
        except DistutilsFileError:
            # If it's a file 
            copy_file(from_path, to_path)
